#!/usr/bin/env ruby
###########################################################################
###########################################################################
##  Copyright The Resistance (C) 1995-2011                               ##
##  Author: blass                                                        ##
##  blass[at]sdf.lonestar.org                                            ##
##  Badass 0.2 Beta                                                      ##
##  This file is part of Badass.                                         ##
##  Badass is free software: you can redistribute it and/or modify       ##
##  it under the terms of the GNU General Public License as published by ##
##  the Free Software Foundation, either version 3 of the License, or    ##
##  (at your option) any later version.                                  ##
##  Badass is distributed in the hope that it will be useful,            ##
##  but WITHOUT ANY WARRANTY; without even the implied warranty of       ##
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        ##
##  GNU General Public License for more details.                         ##
##  You should have received a copy of the GNU General Public License    ##
##  along with Badass. If not, see http://www.gnu.org/licenses/          ##
###########################################################################
###########################################################################

module Racket
  module L4
    # User Datagram Protocol: UDP
    #
    # RFC768 (http://www.faqs.org/rfcs/rfc768.html)
    class UDP < RacketPart
      # Source Port
      unsigned :src_port, 16
      # Destination Port
      unsigned :dst_port, 16
      # Datagram Length
      unsigned :len, 16
      # Checksum
      unsigned :checksum, 16
      # Payload
      rest :payload
      
      # Check the checksum for this UDP datagram
      def checksum?(src_ip, dst_ip)
        self.checksum == 0 || (self.checksum == compute_checksum(src_ip, dst_ip))
      end
      
      # Compute and set the checksum for this UDP datagram
      def checksum!(src_ip, dst_ip)
        # set the checksum to 0 for usage in the pseudo header...
        self.checksum = 0
        self.checksum = compute_checksum(src_ip, dst_ip)
      end
      
      # Fix this packet up for proper sending.  Sets the length
      # and checksum properly.
      def fix!(src_ip, dst_ip)
        self.len = self.class.bit_length/8 + self.payload.length
        self.checksum!(src_ip, dst_ip)
      end
      
      def initialize(*args)
        super
        @autofix = false
      end 
      
      private
      # Compute the checksum for this UDP datagram
      def compute_checksum(src_ip, dst_ip)
        # pseudo header used for checksum calculation as per RFC 768 
        pseudo = [L3::Misc.ipv42long(src_ip), L3::Misc.ipv42long(dst_ip), 17, self.payload.length + self.class.bit_length/8 ]
        header = [self.src_port, self.dst_port, self.payload.length + self.class.bit_length/8, 0, self.payload]
        L3::Misc.checksum((pseudo << header).flatten.pack("NNnnnnnna*"))
      end
      
    end
  end
end
# vim: set ts=2 et sw=2:
